==> armadillo_0.6.10.orig.tar.gz: armadillo-0.6.10/include/armadillo_bits/Mat_proto.hpp <==
// Copyright (C) 2009 NICTA
// 
// Authors:
// - Conrad Sanderson (conradsand at ieee dot org)
// 
// This file is part of the Armadillo C++ library.
// It is provided without any warranty of fitness
// for any purpose. You can redistribute this file
// and/or modify it under the terms of the GNU
// Lesser General Public License (LGPL) as published
// by the Free Software Foundation, either version 3
// of the License or (at your option) any later version.
// (see http://www.opensource.org/licenses for more info)


//! \addtogroup Mat
//! @{

//! Dense matrix class

template<typename eT>
class Mat : public Base<eT, Mat<eT> >
  {
  public:
  
  typedef eT elem_type;  //!< the type of elements stored in the matrix
  
  typedef typename get_pod_type<elem_type>::pod_type pod_type;
  //!< if eT is std::complex, pod_type is the underlying type used by std::complex.
  //!< otherwise pod_type is the same as elem_type
  
  
  const u32 n_rows;     //!< number of rows in the matrix (read-only)
  const u32 n_cols;     //!< number of columns in the matrix (read-only)
  const u32 n_elem;     //!< number of elements in the matrix (read-only)
  
  arma_aligned const eT* const mem;  //!< pointer to memory used by the matrix (memory is read-only)
  
  protected:
  arma_aligned eT mem_local[ 16 ];
  
  
  public:
  
  inline ~Mat();
  inline  Mat();
  
  inline            Mat(const u32 in_rows, const u32 in_cols);
  inline void  set_size(const u32 in_rows, const u32 in_cols);
  
  inline                  Mat(const char* text);
  inline const Mat& operator=(const char* text);
  inline                  Mat(const std::string& text);
  inline const Mat& operator=(const std::string& text);
  
  inline Mat(const eT* aux_mem, const u32 aux_n_rows, const u32 aux_n_cols);
  
  arma_inline const Mat&  operator=(const eT val);
  arma_inline const Mat& operator+=(const eT val);
  arma_inline const Mat& operator-=(const eT val);
  arma_inline const Mat& operator*=(const eT val);
  arma_inline const Mat& operator/=(const eT val);
  
  inline                   Mat(const Mat& m);
  inline const Mat&  operator=(const Mat& m);
  inline const Mat& operator+=(const Mat& m);
  inline const Mat& operator-=(const Mat& m);
  inline const Mat& operator*=(const Mat& m);
  inline const Mat& operator%=(const Mat& m);
  inline const Mat& operator/=(const Mat& m);

  template<typename T1, typename T2>
  inline explicit Mat(const Base<pod_type,T1>& A, const Base<pod_type,T2>& B);

  inline                   Mat(const subview<eT>& X);
  inline const Mat&  operator=(const subview<eT>& X);
  inline const Mat& operator+=(const subview<eT>& X);
  inline const Mat& operator-=(const subview<eT>& X);
  inline const Mat& operator*=(const subview<eT>& X);
  inline const Mat& operator%=(const subview<eT>& X);
  inline const Mat& operator/=(const subview<eT>& X);
  
  //inline explicit          Mat(const diagview<eT>& X);
  inline                   Mat(const diagview<eT>& X);
  inline const Mat&  operator=(const diagview<eT>& X);
  
  arma_inline       subview_row<eT> row(const u32 row_num);
  arma_inline const subview_row<eT> row(const u32 row_num) const;
  
  arma_inline       subview_col<eT> col(const u32 col_num);
  arma_inline const subview_col<eT> col(const u32 col_num) const;
  
  arma_inline       subview<eT> rows(const u32 in_row1, const u32 in_row2);
  arma_inline const subview<eT> rows(const u32 in_row1, const u32 in_row2) const;
  
  arma_inline       subview<eT> cols(const u32 in_col1, const u32 in_col2);
  arma_inline const subview<eT> cols(const u32 in_col1, const u32 in_col2) const;
  
  arma_inline       subview<eT> submat(const u32 in_row1, const u32 in_col1, const u32 in_row2, const u32 in_col2);
  arma_inline const subview<eT> submat(const u32 in_row1, const u32 in_col1, const u32 in_row2, const u32 in_col2) const;

  arma_inline       diagview<eT> diag(const s32 in_id = 0);
  arma_inline const diagview<eT> diag(const s32 in_id = 0) const;
    
  inline void swap_rows(const u32 in_row1, const u32 in_row2);
  inline void swap_cols(const u32 in_col1, const u32 in_col2);
  
  template<typename T1, typename op_type> inline                   Mat(const Op<T1, op_type>& X);
  template<typename T1, typename op_type> inline const Mat&  operator=(const Op<T1, op_type>& X);
  template<typename T1, typename op_type> inline const Mat& operator+=(const Op<T1, op_type>& X);
  template<typename T1, typename op_type> inline const Mat& operator-=(const Op<T1, op_type>& X);
  template<typename T1, typename op_type> inline const Mat& operator*=(const Op<T1, op_type>& X);
  template<typename T1, typename op_type> inline const Mat& operator%=(const Op<T1, op_type>& X);
  template<typename T1, typename op_type> inline const Mat& operator/=(const Op<T1, op_type>& X);
  
  template<typename T1, typename T2, typename glue_type> inline                   Mat(const Glue<T1, T2, glue_type>& X);
  template<typename T1, typename T2, typename glue_type> inline const Mat&  operator=(const Glue<T1, T2, glue_type>& X);
  template<typename T1, typename T2, typename glue_type> inline const Mat& operator+=(const Glue<T1, T2, glue_type>& X);
  template<typename T1, typename T2, typename glue_type> inline const Mat& operator-=(const Glue<T1, T2, glue_type>& X);
  template<typename T1, typename T2, typename glue_type> inline const Mat& operator*=(const Glue<T1, T2, glue_type>& X);
  template<typename T1, typename T2, typename glue_type> inline const Mat& operator%=(const Glue<T1, T2, glue_type>& X);
  template<typename T1, typename T2, typename glue_type> inline const Mat& operator/=(const Glue<T1, T2, glue_type>& X);
  
  
  arma_inline eT& operator[] (const u32 i);
  arma_inline eT  operator[] (const u32 i) const;
  arma_inline eT& operator() (const u32 i);
  arma_inline eT  operator() (const u32 i) const;
  
  arma_inline eT& at         (const u32 in_row, const u32 in_col);
  arma_inline eT  at         (const u32 in_row, const u32 in_col) const;
  arma_inline eT& operator() (const u32 in_row, const u32 in_col);
  arma_inline eT  operator() (const u32 in_row, const u32 in_col) const;

  arma_inline const Mat& operator++();
  arma_inline void       operator++(int);

  arma_inline const Mat& operator--();
  arma_inline void       operator--(int);

  arma_inline bool is_vec() const;
  arma_inline bool is_square() const;
  arma_inline bool is_finite() const;

  arma_inline       eT* colptr(const u32 in_col);
  arma_inline const eT* colptr(const u32 in_col) const;
  
  arma_inline       eT* memptr();
  arma_inline const eT* memptr() const;

  inline void print(const std::string extra_text = "") const;

  inline void fill(const eT val);
  inline void zeros();
  inline void zeros(const u32 in_rows, const u32 in_cols);
  
  inline void reset();
  
  inline void save(const std::string name, const file_type type = arma_binary) const;
  inline void load(const std::string name, const file_type type = auto_detect);
  
  
  protected: 
  
  inline void init(const u32 in_rows, const u32 in_cols);
  inline void init(const std::string& text);
  inline void init(const Mat& x);
  };



class Mat_aux
  {
  public:

  template<typename eT> arma_inline static void prefix_pp(Mat<eT>& x);
  template<typename T>  arma_inline static void prefix_pp(Mat< std::complex<T> >& x);
  
  template<typename eT> arma_inline static void postfix_pp(Mat<eT>& x);
  template<typename T>  arma_inline static void postfix_pp(Mat< std::complex<T> >& x);
  
  template<typename eT> arma_inline static void prefix_mm(Mat<eT>& x);
  template<typename T>  arma_inline static void prefix_mm(Mat< std::complex<T> >& x);
  
  template<typename eT> arma_inline static void postfix_mm(Mat<eT>& x);
  template<typename T>  arma_inline static void postfix_mm(Mat< std::complex<T> >& x);
  };



//! @}

==> armadillo_0.8.0.orig.tar.gz: armadillo-0.8.0/include/armadillo_bits/Mat_proto.hpp <==
// Copyright (C) 2009 NICTA
// 
// Authors:
// - Conrad Sanderson (conradsand at ieee dot org)
// 
// This file is part of the Armadillo C++ library.
// It is provided without any warranty of fitness
// for any purpose. You can redistribute this file
// and/or modify it under the terms of the GNU
// Lesser General Public License (LGPL) as published
// by the Free Software Foundation, either version 3
// of the License or (at your option) any later version.
// (see http://www.opensource.org/licenses for more info)


//! \addtogroup Mat
//! @{

//! Dense matrix class

template<typename eT>
class Mat : public Base< eT, Mat<eT> >
  {
  public:
  
  typedef eT elem_type;  //!< the type of elements stored in the matrix
  
  typedef typename get_pod_type<elem_type>::pod_type pod_type;
  //!< if eT is std::complex, pod_type is the underlying type used by std::complex.
  //!< otherwise pod_type is the same as elem_type
  
  
  const u32  n_rows;      //!< number of rows in the matrix (read-only)
  const u32  n_cols;      //!< number of columns in the matrix (read-only)
  const u32  n_elem;      //!< number of elements in the matrix (read-only)
  const bool use_aux_mem; //!< true if externally managed memory is being used (read-only)
  
  arma_aligned const eT* const mem;  //!< pointer to the memory used by the matrix (memory is read-only)
  
  protected:
  arma_aligned eT mem_local[ 16 ];
  
  
  public:
  
  inline ~Mat();
  inline  Mat();
  
  inline Mat(const u32 in_rows, const u32 in_cols);
  
  inline                  Mat(const char*        text);
  inline const Mat& operator=(const char*        text);
  inline                  Mat(const std::string& text);
  inline const Mat& operator=(const std::string& text);
  
  inline Mat(      eT* aux_mem, const u32 aux_n_rows, const u32 aux_n_cols, const bool copy_aux_mem = true);
  inline Mat(const eT* aux_mem, const u32 aux_n_rows, const u32 aux_n_cols);
  
  arma_inline const Mat&  operator=(const eT val);
  arma_inline const Mat& operator+=(const eT val);
  arma_inline const Mat& operator-=(const eT val);
  arma_inline const Mat& operator*=(const eT val);
  arma_inline const Mat& operator/=(const eT val);
  
  inline                   Mat(const Mat& m);
  inline const Mat&  operator=(const Mat& m);
  inline const Mat& operator+=(const Mat& m);
  inline const Mat& operator-=(const Mat& m);
  inline const Mat& operator*=(const Mat& m);
  inline const Mat& operator%=(const Mat& m);
  inline const Mat& operator/=(const Mat& m);

  template<typename T1, typename T2>
  inline explicit Mat(const Base<pod_type,T1>& A, const Base<pod_type,T2>& B);

  inline                   Mat(const subview<eT>& X);
  inline const Mat&  operator=(const subview<eT>& X);
  inline const Mat& operator+=(const subview<eT>& X);
  inline const Mat& operator-=(const subview<eT>& X);
  inline const Mat& operator*=(const subview<eT>& X);
  inline const Mat& operator%=(const subview<eT>& X);
  inline const Mat& operator/=(const subview<eT>& X);

  //inline explicit          Mat(const subview_cube<eT>& X);
  inline                   Mat(const subview_cube<eT>& X);
  inline const Mat&  operator=(const subview_cube<eT>& X);
  inline const Mat& operator+=(const subview_cube<eT>& X);
  inline const Mat& operator-=(const subview_cube<eT>& X);
  inline const Mat& operator*=(const subview_cube<eT>& X);
  inline const Mat& operator%=(const subview_cube<eT>& X);
  inline const Mat& operator/=(const subview_cube<eT>& X);

  
  //inline explicit          Mat(const diagview<eT>& X);
  inline                   Mat(const diagview<eT>& X);
  inline const Mat&  operator=(const diagview<eT>& X);
  
  arma_inline       subview_row<eT> row(const u32 row_num);
  arma_inline const subview_row<eT> row(const u32 row_num) const;
  
  arma_inline       subview_col<eT> col(const u32 col_num);
  arma_inline const subview_col<eT> col(const u32 col_num) const;
  
  arma_inline       subview<eT> rows(const u32 in_row1, const u32 in_row2);
  arma_inline const subview<eT> rows(const u32 in_row1, const u32 in_row2) const;
  
  arma_inline       subview<eT> cols(const u32 in_col1, const u32 in_col2);
  arma_inline const subview<eT> cols(const u32 in_col1, const u32 in_col2) const;
  
  arma_inline       subview<eT> submat(const u32 in_row1, const u32 in_col1, const u32 in_row2, const u32 in_col2);
  arma_inline const subview<eT> submat(const u32 in_row1, const u32 in_col1, const u32 in_row2, const u32 in_col2) const;

  arma_inline       diagview<eT> diag(const s32 in_id = 0);
  arma_inline const diagview<eT> diag(const s32 in_id = 0) const;
    
  inline void swap_rows(const u32 in_row1, const u32 in_row2);
  inline void swap_cols(const u32 in_col1, const u32 in_col2);
  
  template<typename T1, typename op_type> inline                   Mat(const Op<T1, op_type>& X);
  template<typename T1, typename op_type> inline const Mat&  operator=(const Op<T1, op_type>& X);
  template<typename T1, typename op_type> inline const Mat& operator+=(const Op<T1, op_type>& X);
  template<typename T1, typename op_type> inline const Mat& operator-=(const Op<T1, op_type>& X);
  template<typename T1, typename op_type> inline const Mat& operator*=(const Op<T1, op_type>& X);
  template<typename T1, typename op_type> inline const Mat& operator%=(const Op<T1, op_type>& X);
  template<typename T1, typename op_type> inline const Mat& operator/=(const Op<T1, op_type>& X);
  
  template<typename T1, typename T2, typename glue_type> inline                   Mat(const Glue<T1, T2, glue_type>& X);
  template<typename T1, typename T2, typename glue_type> inline const Mat&  operator=(const Glue<T1, T2, glue_type>& X);
  template<typename T1, typename T2, typename glue_type> inline const Mat& operator+=(const Glue<T1, T2, glue_type>& X);
  template<typename T1, typename T2, typename glue_type> inline const Mat& operator-=(const Glue<T1, T2, glue_type>& X);
  template<typename T1, typename T2, typename glue_type> inline const Mat& operator*=(const Glue<T1, T2, glue_type>& X);
  template<typename T1, typename T2, typename glue_type> inline const Mat& operator%=(const Glue<T1, T2, glue_type>& X);
  template<typename T1, typename T2, typename glue_type> inline const Mat& operator/=(const Glue<T1, T2, glue_type>& X);
  
  
  arma_inline eT& operator[] (const u32 i);
  arma_inline eT  operator[] (const u32 i) const;
  arma_inline eT& operator() (const u32 i);
  arma_inline eT  operator() (const u32 i) const;
  
  arma_inline eT& at         (const u32 in_row, const u32 in_col);
  arma_inline eT  at         (const u32 in_row, const u32 in_col) const;
  arma_inline eT& operator() (const u32 in_row, const u32 in_col);
  arma_inline eT  operator() (const u32 in_row, const u32 in_col) const;
  
  arma_inline const Mat& operator++();
  arma_inline void       operator++(int);
  
  arma_inline const Mat& operator--();
  arma_inline void       operator--(int);
  
  arma_inline bool is_vec() const;
  arma_inline bool is_square() const;
  arma_inline bool is_finite() const;
  
  arma_inline       eT* colptr(const u32 in_col);
  arma_inline const eT* colptr(const u32 in_col) const;
  
  arma_inline       eT* memptr();
  arma_inline const eT* memptr() const;
  
  inline void print(const std::string extra_text = "") const;
  inline void print(std::ostream& user_stream, const std::string extra_text = "") const;

  inline void print_trans(const std::string extra_text = "") const;
  inline void print_trans(std::ostream& user_stream, const std::string extra_text = "") const;

  inline void raw_print(const std::string extra_text = "") const;
  inline void raw_print(std::ostream& user_stream, const std::string extra_text = "") const;

  inline void raw_print_trans(const std::string extra_text = "") const;
  inline void raw_print_trans(std::ostream& user_stream, const std::string extra_text = "") const;

  template<typename eT2>
  inline void copy_size(const Mat<eT2>& m);
  
  inline void  set_size(const u32 in_rows, const u32 in_cols);
  
  inline void fill(const eT val);
  
  inline void zeros();
  inline void zeros(const u32 in_rows, const u32 in_cols);
  
  inline void ones();
  inline void ones(const u32 in_rows, const u32 in_cols);
  
  inline void reset();
  
  inline void save(const std::string name, const file_type type = arma_binary) const;
  inline void load(const std::string name, const file_type type = auto_detect);
  
  
  protected: 
  
  inline void init(const u32 in_rows, const u32 in_cols);
  inline void init(const std::string& text);
  inline void init(const Mat& x);
  
  inline Mat(const char junk, const eT* aux_mem, const u32 aux_n_rows, const u32 aux_n_cols);
  
  friend class Cube<eT>;
  };



class Mat_aux
  {
  public:

  template<typename eT> arma_inline static void prefix_pp(Mat<eT>& x);
  template<typename T>  arma_inline static void prefix_pp(Mat< std::complex<T> >& x);
  
  template<typename eT> arma_inline static void postfix_pp(Mat<eT>& x);
  template<typename T>  arma_inline static void postfix_pp(Mat< std::complex<T> >& x);
  
  template<typename eT> arma_inline static void prefix_mm(Mat<eT>& x);
  template<typename T>  arma_inline static void prefix_mm(Mat< std::complex<T> >& x);
  
  template<typename eT> arma_inline static void postfix_mm(Mat<eT>& x);
  template<typename T>  arma_inline static void postfix_mm(Mat< std::complex<T> >& x);
  };



//! @}
